<?php
/**
 * Helps to fetch and output informations of an App Store App.
 * 
 * You can find some examples at http://geardev.de/appstoreapi.php
 * 
 * @author Rudolf Hattenkofer
 * @version 1.0.1
 */
class App {
	public $age;                 // The rating as iTunes outputs it (4+ etc.)
	public $artwork;             // The artworks as array (60, 100, 512)
	public $description;         // The description
	public $devices;             // Contains an array of supported devices
	public $gamecenter_enabled;  // Boolean, is the app GameCenter enabled?
	public $genre;               // The genre
	public $id;                  // The ID
	public $price;               // The price
	public $rating;              // User rating, decimals for half star ratings (4.5, 3.0)
	public $rating_count;        // Number of user ratings
	public $released;            // The Release date
	public $screenshots;         // Array with all screenshot URLs
	public $seller;              // The Company which sells the App
	public $seller_url;          // Website of the company which sells the App
	public $size;                // The Size, in Bytes
	public $title;               // The Title
	public $url;                 // The iTunes link
	public $version;             // Current version
	public $json;                // What iTunes returns
	
	/** 
	 * Fetches the app data from iTunes and saves it in the variables
	 * 
	 * @param int/string $id        -> ID of the app (type doesn't matter because we add it to a string)
	 * @param string     $country   -> country (e.g. US or DE)
	 * @param string     $language  -> language (e.g. en_us, en or de_de)
	 * (For the last two parameters it's practical to change the defaults below)
	 */
	function __construct( $id = false, $country = 'US', $language = 'en_us' ) {
		/**
		 * I think we don't need to check whether the app exists or not, you 
		 * would notice it because every variable will be NULL ;)
		 * We just start to fetch the data from iTunes and decode the results
		 * to an array.
		 */
		$this->id   = $id;
		$this->json = file_get_contents( 'http://ax.phobos.apple.com.edgesuite.net/WebObjects/MZStoreServices.woa/wa/wsLookup?country='.$country.'&lang='.$language.'&id='.$id );
		$itunes     = json_decode( $this->json, true );
		/**
		 * We would have to type $itunes['results'][0] over and over again, so
		 * we overwrite $itunes with the actual content (Just the first, because
		 * we look after an app with a unique ID)
		 */
		$itunes     = $itunes['results'][0];
		
		// Now we save everything in the classes variables
		$this->age                = $itunes['trackContentRating'];
		$this->artwork            = array( 
										'60'  => $itunes['artworkUrl60'],
										'100' => $itunes['artworkUrl100'],
										'512' => $itunes['artworkUrl512'],
									);
		$this->description        = $itunes['description'];
		$this->devices            = $itunes['supportedDevices'];
		$this->gamecenter_enabled = $itunes['isGameCenterEnabled'];
		$this->genre              = $itunes['primaryGenreName'];
		$this->price              = $itunes['price'];
		$this->rating             = $itunes['averageUserRating'];
		$this->rating_count       = $itunes['userRatingCount'];
		$this->released           = $itunes['releaseDate'];
		$this->screenshots        = array(
										'iphone' => $itunes['screenshotUrls'],
										'ipad'   => $itunes['ipadScreenshotUrls'],
									);
		$this->seller             = $itunes['artistName'];
		$this->seller_url         = $itunes['sellerUrl'];
		$this->size               = $itunes['fileSizeBytes'];
		$this->title              = $itunes['trackCensoredName'];
		$this->url                = $itunes['trackViewUrl'];
		$this->version            = $itunes['version'];
	}
	
	/**
	 * Returns the age rating of the app
	 *
	 * @param bool $original  -> If true, returns as iTunes outputs it (4+ etc.). If false, returns without the '+'
	 */
	function age( $original = true ) {
		if ( $original )
			return $this->age;
		
		return preg_replace('/\+/', '', $this->age);
	}
	
	/**
	 * Returns the artwork in the selected size
	 * TODO: Eventually crop and save in a given directory in a future version
	 * 
	 * @param int $size  -> Size of the Artwork, currently 60, 100 or 512
	 */
	function artwork( $size = 60 ) {
		return $this->artwork[$size];
	}
	
	/**
	 * Returns the description
	 */
	function description() {
		return $this->description;
	}
	
	/**
	 * Returns the supported devices and (optionally) if the selected device is supported
	 * 
	 * @param string $device  -> Checks if the selected device is supported
	 */
	function devices( $device = false ) {
		if ( $device == false )
			return $this->devices;
		
		if ( 'all' == $this->devices )  // If all devices are supported, just return true
			return true;
		
		$found = false;
		foreach ( $this->devices as $current_device ) {
			/**
			 * Checks for each value of $this->devices if it matches the search
			 * string. Before that, the strings are converted to lower case, so
			 * that e.g. 'ipad' also returns true.
			 * Also searches for the string with strpos() instead of matching, 
			 * because e.g. the iPad is separated into iPad3G and iPadWiFi.
			 */
			if( strpos( strtolower( $current_device ), strtolower( $device ) ) !== false )
				$found = true;
		}
		return $found;
	}
	
	/**
	 * Returns if the Game supports GameCenter
	 */
	function gamecenterEnabled() {
		return $this->gamecenter_enabled;
	}
	
	/**
	 * Returns the Genre
	 */
	function genre() {
		return $this->genre;
	}
	
	/**
	 * Returns the ID (not quite useful, but for completeness)
	 */
	function id() {
		return $this->id;
	}
	
	/**
	 * Returns the price
	 * 
	 * @param string $currency  -> The symbol which is placed after the price
	 * @param string $decimal   -> Could be 'comma' or 'point'
	 * @param string $free      -> String returned if the app is free
	 * @param bool   $space     -> If true, a space will be added between price and currency
	 */
	function price( $currency = '$', $decimal = 'point', $free = 'Free', $space = false ) {
		if ( $this->price == 0.00 )
			return $free;
		
		if ( 'point' == $decimal )
			$price = $this->price;
		else {
			$price = preg_replace('/./', ',', $this->price);
		}
		
		if ( $space )
			$price = $price.' ';
		
		return $price.$currency;
	}
	
	/**
	 * Returns the average user rating, decimals for half star ratings (4.5, 3.0)
	 */
	function rating() {
		return $this->rating;
	}
	
	/**
	 * Returns the number of User Ratings
	 */
	function rating_count() {
		return $this->rating_count;
	}
	
	/**
	 * Returns the Release Date
	 * 
	 * @param string $format  -> example: M.D.Y H:m:S (that are all the replaced strings)
	 */
	function released( $format = false ) {
		if ( $format == false )
			return $this->released;
		
		$date    = $this->released;
		$year    = substr($date, 0,  4);
		$month   = substr($date, 5,  2);
		$day     = substr($date, 8,  2);
		$hour    = substr($date, 11, 2);
		$minute  = substr($date, 14, 2);
		$second  = substr($date, 17, 2);
		$search  = array( '/Y/', '/M/',  '/D/', '/H/', '/m/',   '/S/' );
		$replace = array( $year, $month, $day,  $hour, $minute, $second);
		return preg_replace($search, $replace, $format);
	}
	
	/**
	 * Returns an Array of Screenshots
	 * 
	 * @param string $device  -> could be 'ipad' or 'iphone'
	 */
	function screenshots( $device = false ) {
		if ( ! $device )
			return $this->screenshots;
		
		if ( 'iphone' == $device )
			return $this->screenshots['iphone'];
		
		if ( 'ipad'   == $device)
			return $this->screenshots['ipad'];
	}
	
	/**
	 * Returns the company which sells the app
	 */
	function seller() {
		return $this->seller;
	}
	
	/**
	 * Returns the website URL of the company which sells the app
	 */
	function sellerUrl() {
		return $this->seller_url;
	}
	
	/**
	 * Returns the size of the app
	 * 
	 * @param bool   $shorten  -> If true, returns e.g. 5.4MB (Up to GB, more is unlikely)
	 * @param string $decimal  -> could be 'point' or 'comma'
	 * @param bool   $space    -> If true, returns e.g. 5.4 MB
	 */
	function size( $shorten = true, $decimal = 'point', $space = false ) {
		if ( $shorten == false )  // If we don't have to shorten, there is no space and no decimals
			return $this->size;
		
		$size = $this->size;
		if ($size > 1024) {  // Calculate the "right" size
			$size = round($size / 1024);
			$unit = 'KB';
			if ($size > 1024) {
				$size = round($size / 1024, 1);
				$unit = 'MB';
				if ($size > 1024) {
					$size = round($size / 1024, 1);
					$unit = 'GB';
				}
			}
		} else {
			$unit = 'Byte';
		}
		
		if ( 'comma' == $decimal )  // Replace point with comma if wished
			$size = preg_replace('/\./', ',', $size);
			
		if ( $space == true )  // Add a space and abort if wished
			return $size.' '.$unit;
			
		return $size.$unit;  // Just add the unit and return
		
		// TODO: Eventually add option to set the unit
	}
	
	/**
	 * Returns the title of the app
	 * 
	 * @param int    $shorten  -> Shortens the title to the given number of characters
	 * @param string $after    -> Will be added at the end if title is longer than $shorten
	 */
	function title( $shorten = false, $after = '...' ) {
		if ( $shorten == false )
			return $this->title;
		
		$shorten --; // User starts counting with 1, but we do with 0
		
		$title_shortened = substr( $this->title, 0, $shorten + 1 );
		
		if ( strlen( $this->title ) > $shorten ) {
			/**
			 * Up:     Checks if the shortened title is longer than the requested length
			 * Míddle: Checks if the last character is a space, removes if true
			 * Bottom: Adds the $after string to the title
			 */
			if ( strrpos( $title_shortened, ' ' ) == $shorten )
				$title_shortened = substr( $this->title, 0, $shorten );
			
			$title_shortened .= $after;
		}
		
		return $title_shortened;
	}
	
	/**
	 * Returns the iTunes link
	 */
	function url() {
		return $this->url;
	}
	
	/**
	 * Returns the version number
	 */
	function version() {
		return $this->version;
	}
	
	/**
	 * Outputs the app information in HTML
	 * 
	 * @param $shadow      -> The URL of the shadow (if nothing is given, the artwork won't have rounded eges)
	 * @param $linkname    -> The Text of the Download Link
	 * @param $sellertext  -> The Text before the seller
	 * @param $ratingtext  -> The Text before the rating
	 * @param $pricentext  -> The Text before the price
	 * @param $freetext    -> The Text inserted if the app is free
	 * @param $currency    -> The currency
	 * @param $decimal     -> could be 'point' or 'comma'
	 * (It's practical to modify the default values below if they don't match your language)
	 */
	function output( $shadow = 'shadow.png', $linktext = 'Download', $sellertext = 'Seller: ', $ratingtext = 'Rating: ', $pricentext = 'Price: ', $freetext = 'Free', $currency = '$', $decimal = 'point' ) {
		if ( $this->sellerUrl() ) {
			$seller = '<a href="'.$this->sellerUrl().'">'.$this->seller().'</a>';
		} else {
			$seller = $this->seller();
		}
		
		/**
		 * The <style>...</style> is defined in another file to keep this one clean.
		 * The two links (seller & download) aren't styled because you may make them
		 * look like all the links on your site.
		 * Also you have to take care of proper margin between two app outputs because
		 * otherwise the shadows are too close together.
		 */
		include 'style.php';
		global $appstylesoutputted;
		if ( ! $appstylesoutputted ) {   // We only want to output the styles once, so we check that
			$appstylesoutputted = true;  // and set our uniquie variable to true
			echo $style;
		}
		
		$code = '<div class="app">'.
				'	<div class="iconcontainer">'.
				'		<span class="mask"></span>'.
				'		<img src="'.$this->artwork(60).'" alt="">'.
				'	</div>'.
				'	<span class="title">' .$this->title(60).'</span>'.
				'	<span class="seller"><small>'.$sellertext.'</small>'.$seller.'</span>'.
				'	<span class="rating"><small>'.$ratingtext.'</small>'.$this->age().'</span>'.
				'	<span class="price">'.
				'		<small>'.$pricentext.'</small>' .$this->price( $currency, $decimal, $freetext ).
				'		<a href="'.$this->url().'">'.$linktext.'</a>'.
				'	</span>'.
				'</div>';
		
		echo $code."\n";  // Linke break to the end, so that your code stays more or less clean (The code itself hasn't any line breaks...)
		return $code;
	}
	
	/**
	 * Just a synonym for output(), if you like info() more.
	 */
	function info( $shadow = 'shadow.png', $linktext = 'Download', $sellertext = 'Seller: ', $ratingtext = 'Rating: ', $pricentext = 'Price: ', $freetext = 'Free', $curreny = '$', $decimal = 'point' ) {
		$this->output( $shadow, $linktext, $sellertext, $ratingtext, $pricentext, $freetext, $curreny, $decimal );
	}
}
?>