<?php

require_once 'application/Account/Account.php';

class Contact_Controller extends Account_Controller {

	public function indexGet($filter=NULL) {
		$this->_verifyUserAuth();

		try {
			$user = vista_get_user();

			if ( true === $user->isConsumer() ) {
				throw new Vista_Controller_Exception(_('Only business members can manage their friends.'));
			}

			$list = $user->getContactList();
			if ( FILTER_PENDING == $filter ) {
				$list->filter('status = ?', USER_CONTACT_NEW);
			}

			$this->list = $list;
			$this->renderLayout('list');
		} catch ( Vista_Controller_Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$this->_showDisabledError($e->getMessage(), 'account/index');
		} catch ( Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$this->_showDisabledError();
		}

		return true;
	}

	public function addPost($profile_id, $related, $status=USER_CONTACT_NEW, $echo=true) {
		$this->_verifyUserAuth(true);
		$this->setLayout(NULL);

		try {
			$related = intval($related);

			$user = vista_get_user();
			$profile = PV::getModel('user/profile', $profile_id);

			if ( false === $user->isBusiness() ) {
				throw new Vista_Controller_Exception(_('Only business members can have friends.'));
			}

			if ( false === $profile->isBusiness() ) {
				throw new Vista_Controller_Exception(_('Only business members can be friends.'));
			}

			if ( false === $profile->enabled() ) {
				throw new Vista_Controller_Exception(_('The user you attempted to befriend is not enabled.'));
			}

			/* Ensure these people do not already have any relationship. */
			$result_related = PV::getDb()->select()
				->from('user_contact')
				->where('from_user_id = ?', $user->getId())
				->where('to_user_id = ?', $profile->getId())
				->query();
			if ( 0 != $result_related->numRows() ) {
				throw new Vista_Controller_Exception(_('You already have a relationship with this member.'));
			}

			PV::getModel('user/contact')->setFromUserId($user->getId())
				->setToUserId($profile->getId())
				->setRelated($related)
				->setComment(NULL)
				->setStatus($status)
				->write();

			$status_text = _('Friend successfully added.');

			/* Only let the user know on new pending contacts, not on reciprocal contacts. */
			if ( $status != USER_CONTACT_APPROVED ) {
				$status_text = _('Friend successfully added for approval.');
				PV::getEmail()->send('account-pending-contact', $profile->getEmailAddress(), array(
					'firstname' => $profile->getFirstname()
					)
				);
			}
		} catch ( Vista_Controller_Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$status_text = $e->getMessage();
		} catch ( Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$status_text = _('An error occurred when adding your friend.');
		}

		if ( true === $echo ) {
			echo $status_text;
		}

		return true;
	}

	public function approvePost($contact_id, $echo=true) {
		$this->_verifyUserAuth(true);
		$this->setLayout(NULL);

		try {
			$user = vista_get_user();
			$related = intval($this->getParam('related'));

			$this->_setStatus($contact_id, $related, USER_CONTACT_APPROVED);

			$contact = PV::getModel('user/contact', $contact_id);
			$this->addPost($contact->getFromUserId(), $related, USER_CONTACT_APPROVED, false);

			$status_text = _('Your friend was successfully updated.');
		} catch ( Vista_Controller_Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$status_text = $e->getMessage();
		} catch ( Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$status_text = _('An error occurred when approving your friend.');
		}

		if ( true === $echo ) {
			echo $status_text;
		}

		return true;
	}

	public function declinePost($contact_id, $echo=true) {
		$this->_verifyUserAuth(true);
		$this->setLayout(NULL);

		try {
			$this->_setStatus($contact_id, 0, USER_CONTACT_DECLINED);
			$status_text = _('The person was successfully declined as a friend.');
		} catch ( Vista_Controller_Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$status_text = $e->getMessage();
		} catch ( Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$status_text = _('An error occurred when declining your friend.');
		}

		if ( true === $echo ) {
			echo $status_text;
		}

		return true;
	}

	public function deletePost($contact_id, $echo=true) {
		$this->_verifyUserAuth(true);
		$this->setLayout(NULL);

		try {
			$user = vista_get_user();
			$contact = PV::getModel('user/contact', $contact_id);

			if ( false === $user->isBusiness() ) {
				throw new Vista_Controller_Exception(_('Only business accounts can delete friends.'));
			}

			if ( false === $contact->exists() ) {
				throw new Vista_Controller_Exception(_('That friend does not exist.'));
			}

			if ( false === $contact->belongsTo($user->getId()) ) {
				throw new Vista_Controller_Exception(_('You do not have permission to delete this person as a friend.'));
			}

			$contact->delete();
			$status_text = _('Your friend was successfully removed from your friend list.');
		} catch ( Vista_Controller_Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$status_text = $e->getMessage();
		} catch ( Exception $e ) {
			PV::getLog()->add(LOG_EXCEPTION, $e->getMessage());
			$status_text = _('An error occurred when removing your friend.');
		}

		if ( true === $echo ) {
			echo $status_text;
		}

		return true;
	}

	private function _setStatus($contact_id, $related, $status) {
		$user = vista_get_user();
		$contact = PV::getModel('user/contact', $contact_id);

		if ( false === $user->isBusiness() ) {
			throw new Vista_Controller_Exception(_('Only business accounts can manage friends.'));
		}

		if ( false === $contact->exists() ) {
			throw new Vista_Controller_Exception(_('That friends does not exist.'));
		}

		if ( false === $contact->belongsTo($user->getId()) ) {
			throw new Vista_Controller_Exception(_('You do not have permission to change the status of your friend.'));
		}

		if ( $status == USER_CONTACT_APPROVED ) {
			PV::recordActivity(USER_ACTIVITY_ADD_FRIEND, $contact->getFromUserId(), $contact->getToUserId());
		}

		$contact->setRelated($related)->setStatus($status)->write();

		return true;
	}

}