<?php

require_once 'Object/Email.php';
require_once 'Function/General.php';
require_once 'Function/User.php';
require_once 'Artisan/Functions/Encryption.php';
require_once 'Artisan/Functions/System.php';
require_once 'Artisan/Functions/String.php';
require_once 'Artisan/Interface/Iterable.php';

function __autoload($class) {
	$class_path = str_replace('_', DS, $class) . '.php';
	if ( 0 === strpos($class, 'Artisan_') ) {
		require_once $class_path;
	} else {
		require_once 'Object' . DS . $class_path;
	}
}

class PV {
	private static $_routerConfig = NULL;
	private static $_dbConfig = NULL;
	private static $_localeList = array();
	private static $_commandLine = false;

	public static function setRouterConfig(Artisan_Config $config) {
		self::$_routerConfig = $config;
	}

	public static function setDbConfig(Artisan_Config $config) {
		self::$_dbConfig = $config;
	}

	public static function setLocaleList(array $locale_list) {
		self::$_localeList = $locale_list;
	}

	public static function init($command_line, $include_config=true) {
		$db = new Artisan_Db_Adapter_Mysqli(self::$_dbConfig);
		$db->connect();
		$sql = "SET character_set_results = 'utf8',
			character_set_client = 'utf8',
			character_set_connection = 'utf8',
			character_set_database = 'utf8',
			character_set_server = 'utf8'";
		$db->query($sql);
		Artisan_Registry::push('db', $db);

		self::$_commandLine = $command_line;

		if ( false === $command_line ) {
			$sessDb = new Artisan_Session_Db($db);
			$sessDb->setTableName('session');

			Artisan_Session::get()->setSaveHandler($sessDb);
			Artisan_Session::get()->start();

			User_Session::get()->load();
			$user = User_Factory::build(User_Session::get()->getUserId());
			if ( true === is_object($user) ) {
				Artisan_Registry::push('user', $user);
			}

			self::createToken();

			if ( POST == RM ) {
				$token = asfw_exists_return('token', $_POST);
				if ( false === self::verifyToken($token) ) {
					exit('POST methods are not allowed without the correct token! Token given: ' . $token);
				}
			}
		}

		$email = new Email();
		Artisan_Registry::push('email', $email);

		$log_writer = new Artisan_Log_Writer_Db($db);
		$log_writer->setTable('log');
		Artisan_Log::get()->setWriter($log_writer);

		if ( true === $include_config ) {
			/**
			 * Handle languages and translations.
			 */
			Artisan_Registry::push('locale_list', self::$_localeList);

			$locale = er('locale', $_GET);
			if ( false === empty($locale) && true === isset(self::$_localeList[$locale]) ) {
				setcookie('locale', $locale, time()+(60*60*24*365), '/', COOKIE_DOMAIN);
			} else {
				$locale = er('locale', $_COOKIE);
			}

			if ( true === empty($locale) ) {
				$locale = CONFIG_LOCALE;
			}

			Artisan_Registry::push('locale', $locale);

			setlocale(LC_ALL, $locale);
			bindtextdomain('messages', DIR_LOCALE);
			textdomain('messages');
		}

		/* Set up a list of timezones. */
		$timezone_list = array();
		for ( $i=-11; $i<=8; $i++ ) {
			$date = vista_date(time(), $i);
			$timezone_list[$i] = 'GMT ' . $i . ' (' . $date . ')';
		}

		Artisan_Registry::push('tz', $timezone_list);
	}

	public static function run() {
		$artisanRouter = new Artisan_Controller_Router(self::$_routerConfig);
		echo $artisanRouter->dispatch();
	}

	public static function buildView() {
		$view = new Artisan_Controller_View(self::$_routerConfig->root_dir);
		$view->setIsRewrite(self::$_routerConfig->rewrite)
			->setSiteRoot(self::$_routerConfig->site_root)
			->setSiteRootSecure(self::$_routerConfig->site_root_secure);
		return $view;
	}

	public static function createToken() {
		$_sess = Artisan_Session::get();
		if ( false === $_sess->exists(SESSION_TOKEN) ) {
			$token = mt_rand(1000000, mt_getrandmax());
			$salt = asfw_create_salt();
			$secret_token = asfw_compute_hash($token, $salt);

			$_sess->add(SESSION_TOKEN, $token);
			$_sess->add(SESSION_TOKEN_SECRET, $secret_token);
			$_sess->add(SESSION_TOKEN_SALT, $salt);
		}
	}

	public static function verifyToken($token) {
		$salt = self::getTokenSalt();
		$secret_token = self::getSecretToken();
		$hashed_token = asfw_compute_hash($token, $salt);

		return ( $secret_token === $hashed_token );
	}

	public static function recordActivity($view, $user_id, $profile_id=0, $content=NULL) {
		$user = PV::getModel('user/profile', $user_id);

		$user_activity = PV::getModel('user/activity');
		$user_activity->setDateCreate(time())
			->setUserId((int)$user_id)
			->setProfileId((int)$profile_id)
			->setUsername($user->getUsername())
			->setFullname($user->getFirstname() . ' ' . $user->getLastname())
			->setContentDirectory($user->getContentDirectory())
			->setProfilePictureTn($user->getProfilePictureTn());

		if ( $profile_id > 0 ) {
			$profile = PV::getModel('user/profile', $profile_id);
			if ( true === $profile->enabled() ) {
				$user_activity->setProfileUsername($profile->getUsername())
					->setProfileFullname($profile->getFirstname() . ' ' . $profile->getLastname());
			}
		}

		$user_activity->setView($view)
			->setContent($content)
			->setStatus(STATUS_ENABLED)
			->write();
	}

	public static function getDb() {
		return Artisan_Registry::pop('db');
	}

	public static function getLog() {
		return Artisan_Log::get();
	}

	public static function getEmail() {
		return Artisan_Registry::pop('email');
	}

	public static function getPaginator() {
		return new Paginator();
	}

	public static function getToken() {
		return Artisan_Session::get()->from(SESSION_TOKEN);
	}

	public static function getSecretToken() {
		return Artisan_Session::get()->from(SESSION_TOKEN_SECRET);
	}

	public static function getTokenSalt() {
		return Artisan_Session::get()->from(SESSION_TOKEN_SALT);
	}

	public static function getModel($model, $default=NULL) {
		$model = str_replace('/', '_', $model);
		$model = asfw_rename_model($model);

		$obj = new $model($default);
		return $obj;
	}

	public static function getIterator($model, $primary_key=NULL) {
		$iterator = PV::getModel('pv/iterator', $model);
		$iterator->pkey($primary_key);
		return $iterator;
	}

}