<?php
/*
Plugin Name: Relevant Search
Version: 1.0
Description: Replaces the default WordPress search with relevant results.
Author: Derek Herman
Author URI: http://valendesigns.com/
*/

/**
 * Instantiate class
 *
 * @since 1.0
 */
$relevant_search = new Relevant_Search();

/**
 * Plugin Activation/Deactivation
 *
 * @since 1.0
 */
register_activation_hook( __FILE__, array( &$relevant_search, '_activate' ) );
register_deactivation_hook( __FILE__, array( &$relevant_search, '_deactivate' ) );

/**
 * Required actions
 *
 * @since 1.0
 */
add_action( 'rebuild_index', array( &$relevant_search, '_rebuild_fulltext_index' ) );

/**
 * Required filters
 *
 * @since 1.0
 */
add_filter( 'posts_request', array( &$relevant_search, '_posts_request' ) );

/**
 * Relevant_Search Class
 *
 * @package WordPress
 * @subpackage Relevant Search
 * @author Derek Herman
 */
class Relevant_Search
{
  
  /**
   * Activation
   *
   * @uses _add_fulltext_index()
   * @uses wp_schedule_event()
   *
   * @access private
   * @since 1.0
   *
   * @return void
   */
  function _activate() 
  {
    // builds the initial FULLTEXT index
    $this->_add_fulltext_index();
    // rebuilds the index once daily
    wp_schedule_event( time(), 'daily', 'rebuild_index' );
  }
  
  /**
   * Deactivation
   *
   * @uses _drop_fulltext_index()
   * @uses wp_clear_scheduled_hook()
   *
   * @access private
   * @since 1.0
   *
   * @return void
   */
  function _deactivate() 
  {
    // drops the FULLTEXT index
    $this->_drop_fulltext_index();
    // removes daily rebuild event
    wp_clear_scheduled_hook('rebuild_index');
  }
  
  /**
   * Adds a FULLTEXT index on post_title & post_content
   * after verifying no index currently exists
   *
   * @uses $wpdb
   * @uses query()
   *
   * @access private
   * @since 1.0
   *
   * @return void
   */
  function _add_fulltext_index() 
  {
    global $wpdb;
    if ( !$wpdb->query( "SHOW INDEX FROM {$wpdb->posts} WHERE key_name = 'relevant_search';" ) )
    {
      $wpdb->query( "CREATE FULLTEXT INDEX relevant_search ON {$wpdb->posts} (post_title, post_content);" );
    }
  }
  
  /**
   * Drops the FULLTEXT index
   *
   * @uses $wpdb
   * @uses query()
   *
   * @access private
   * @since 1.0
   *
   * @return void
   */
  function _drop_fulltext_index()
  {
    global $wpdb;
    if ( $wpdb->query( "SHOW INDEX FROM {$wpdb->posts} WHERE key_name = 'relevant_search';" ) )
    {
      $wpdb->query( "ALTER TABLE {$wpdb->posts} DROP INDEX relevant_search;" );
    }
  }
    
  /**
   * Rebuilds the FULLTEXT index
   *
   * @uses $wpdb
   * @uses query()
   *
   * @access private
   * @since 1.0
   *
   * @return void
   */
  function _rebuild_fulltext_index() 
  {
    global $wpdb;
    if ( $wpdb->query( "SHOW INDEX FROM {$wpdb->posts} WHERE key_name = 'relevant_search';" ) )
    {
      $wpdb->query( "ALTER TABLE {$wpdb->posts} DROP INDEX relevant_search;" );
      $wpdb->query( "CREATE FULLTEXT INDEX relevant_search ON {$wpdb->posts} (post_title, post_content);" );
    }
  }
		
  /**
   * Search Request Query
   *
   * @uses is_search()
   * @uses $wpdb
   * @uses $wp_query
   * @uses get_option()
   * @uses get_query_var()
   * @uses remove_filter()
   *
   * @access private
   * @since 1.0
   *
   * @param string $request
   *
   * @return string
   */
  function _posts_request($request) 
  {
    // is search run custom query
    if ( is_search() && isset( $_GET['s'] ) ) 
    { 
      global $wpdb, $wp_query; 
      
      // setup search terms
      if ( $s = $wp_query->query_vars['s'] )
      {
        $search_terms = array();
        
        // adding slashes mess with previous quote grouping, so done after
        $ns = stripslashes( $s );
        if ($sentence = $wp_query->query_vars['sentence'])
        {
          $search_terms = array( $ns );
        } 
        else 
        {
          preg_match_all( '/".*?("|$)|((?<=[\\s",+])|^)[^\\s",+]+/', $ns, $matches );
          $search_terms = array_map( create_function( '$a', 'return trim($a, "\\"\'\\n\\r ");' ), $matches[0] );
        }
      }
      // failed return false
      else
      {
        return false;
      }
      
      // build REGEXP search
      foreach ( $search_terms as $key => $value ) 
      {
        $search_terms[$key] = '[[:<:]]'.$value.'[[:>:]]';
      }
      
      // implode $search_terms array() with sql OR operator
      $regex = implode('|', $search_terms);
      
      // post limit
      $limit = get_option( 'posts_per_page' );
      
      // post offset
      $paged = ( get_query_var( 'paged' ) ) ? get_query_var( 'paged' ) : 1;
      $offset = ( $paged > 1 ) ? ( ( $paged - 1 ) * $limit ) : 0;
      
      // results order by
      $oderby = 'keyword_in_title DESC, mysql_score DESC, posts.post_date DESC';
    	
      // custom search query
      $request = "
        SELECT
          SQL_CALC_FOUND_ROWS posts.*,
          CASE
            WHEN posts.post_title REGEXP '$regex'
              THEN 1
              ELSE 0
            END AS keyword_in_title,
          MATCH ( posts.post_title, posts.post_content )
            AGAINST ( '".addslashes( $s )."' ) AS mysql_score
        FROM
          $wpdb->posts as posts
        WHERE
          posts.post_date_gmt <= now()
          AND posts.post_password = ''
          AND posts.post_status = 'publish'
          AND posts.post_type != 'page'
          AND posts.post_type != 'attachment'
          AND ( posts.post_title REGEXP '$regex'
            OR posts.post_content REGEXP '$regex')
        GROUP BY
          posts.ID
        ORDER BY
          $oderby
        LIMIT 
          $offset, 
          $limit
      ";

      // remove filter to allow for other queries 
      remove_filter( 'posts_request', array(&$this, '_posts_request' ) );
    }
    return $request;
  }
}